@@ -20,8 +20,8 @@
 
 /*
  * $RCSfile: log.h,v $
- * $Revision: 1.4 $
- * $Date: 2009/07/30 00:52:22 $
+ * $Revision: 1.6 $
+ * $Date: 2010/03/18 05:45:02 $
  */
 
 #define SYSLOG_IDENT "samma"
@@ -74,7 +74,7 @@
 
 #define ERR_DIRECTORY_MAKE		"%s: Cannot make directory.(%s): (%s)"
 #define ERR_DIRECTORY_SEARCH		"%s: Cannot search directory.(%s): (%s)"
-#define ERR_DIRECTORY_REMOVE		"%s: Cannot remove directory.(%s): (%s)"
+#define ERR_DIRECTORY_REMOVE		"%s: Cannot remove directory.(%s)"
 #define ERR_DIRECTORY_CHANGE		"%s: Change directory failed.(%s)"
 #define ERR_DIRECTORY_NOT_DIRECTORY	"%s: Not directory.(%s)"
 
@@ -117,6 +117,7 @@
 #define ERR_NULL_FILE_NAME      "%s: samma atlist is NULL."
 #define ERR_STR_CODE		"Invalid str code."
 #define ERR_NULL_VALUE		"Invalid value."
+#define ERR_DEFALT_ENCRYPTION	"Invalid default encryption strings."
 
 int syslog_facility(char *str);
 void init_log();
