@@ -20,8 +20,8 @@
 
 /*
  * $RCSfile: mailzip.h,v $
- * $Revision: 1.2 $
- * $Date: 2009/07/30 03:47:22 $
+ * $Revision: 1.7 $
+ * $Date: 2010/04/10 03:01:02 $
  */
 
 #define IDENT   		"samma"
@@ -35,10 +35,11 @@
 #define ZIPCONV			0
 #define NOZIPCONV		1
 #define XHEADER			"X-SaMMA-Enc"
-#define MBSIZE			1048576
 #define MAX_ARGS		4
 #define MLFICLOSE		0
 #define MLFIABORT		1
+#define BEFOR_RCPTCHECK		0
+#define AFTER_RCPTCHECK		1
 
 /* milter */
 sfsistat mlfi_connect(SMFICTX *, char *, _SOCK_ADDR *);
@@ -82,6 +83,9 @@ struct mlfiPriv
     struct rcptaddr *mlfi_savercpt;
     char            *mlfi_savefrom;
     struct mailinfo *mlfi_minfo;
+    struct rcptinfo *mlfi_passlist;
+    struct rcptinfo *mlfi_rdmpasslist;
+    int		    mlfi_rcptcheck;
 };
 
 pthread_t              child;
