@@ -20,20 +20,22 @@
 
 /*
  * $RCSfile: mailzip_db.c,v $
- * $Revision: 1.1.1.1 $
- * $Date: 2009/07/23 01:42:12 $
+ * $Revision: 1.5 $
+ * $Date: 2010/02/25 03:35:48 $
  */
 
 #include <stdio.h>
+#include <errno.h>
+#include <stdlib.h>
 #include <string.h>
 #include <libdgstr.h>
 
+#include "mailzip_config.h"
 #include "mailzip_db.h"
+#include "global.h"
 
 /*
- * search_db()
- *
- * Check if the data exists in DB.
+ * db_open()
  *
  * Args:
  *  char *path                  string
@@ -75,6 +77,20 @@ db_open(DB **dbp, const char *db_file, DBTYPE db_type)
     return(0);
 }
 
+/*
+ * db_search()
+ *
+ * Args:
+ *  DB 		*dbp
+ *  const char 	*db_file
+ *  char 	*search_string
+ *  char 	**passwd
+ *
+ * Return value:
+ *  FOUND                    Record Found
+ *  NOT_FOUND                Record Not Found
+ *  DB_ERROR                 Error
+ */
 int
 db_search(DB *dbp, const char *db_file, char *search_string, char **passwd)
 {
@@ -110,6 +126,7 @@ db_search(DB *dbp, const char *db_file, char *search_string, char **passwd)
         return(NOT_FOUND);
     // error 
     } else if (ret != 0) {
+        log(ERR_DB, "db_search", "move cursor error");
         return(DB_ERROR);
     }
     // found 
@@ -117,7 +134,7 @@ db_search(DB *dbp, const char *db_file, char *search_string, char **passwd)
 	*passwd = strndup(data.data, data.size);
     }
 
-    // close DB
+    // close cursor
     if (dbcp) {
         t_ret = (dbcp)->c_close(dbcp);
     }
