@@ -20,8 +20,8 @@
 
 /*
  * $RCSfile: sendmail.c,v $
- * $Revision: 1.1.1.1 $
- * $Date: 2009/07/23 01:42:12 $
+ * $Revision: 1.7 $
+ * $Date: 2010/03/23 03:35:34 $
  */
 
 #include <stdio.h>
@@ -75,7 +75,6 @@ child(int fd[2], char **arg_list)
     /* exec file */
     if (execvp(arg_list[0], arg_list) < 0) {
 	log(ERR_EXEC_FILE, "child", arg_list[0], strerror(errno));
-        close(fd[R]);
         exit(EX_TEMPFAIL);      // error
     }
 }
@@ -95,7 +94,8 @@ static int
 parent(int fd[2], struct mailzip *mz)
 {
     FILE *fp;
-    GMimeStream *w_stream;
+    GMimeStream *w_stream, *f_stream;
+    GMimeFilter *filter;
 
     close(fd[R]);
 
@@ -106,14 +106,24 @@ parent(int fd[2], struct mailzip *mz)
     }
 
     pthread_mutex_lock(&gmime_lock);
+
+    /* create write stream */
     w_stream = g_mime_stream_file_new(fp);
-    pthread_mutex_unlock(&gmime_lock);
 
-    g_mime_message_write_to_stream(mz->message, w_stream);
-    g_mime_stream_flush(w_stream);
+    /* create CFLF filter */
+    filter = g_mime_filter_crlf_new(TRUE, FALSE);
 
-    pthread_mutex_lock(&gmime_lock);
-    g_object_unref(w_stream);
+    /* set CFLF filter */
+    f_stream = g_mime_stream_filter_new_with_stream(w_stream);
+    g_object_unref (w_stream);
+
+    g_mime_stream_filter_add((GMimeStreamFilter *)f_stream, filter);
+
+    g_mime_message_write_to_stream(mz->message, f_stream);
+    g_mime_stream_flush(f_stream);
+
+    g_object_unref(filter);
+    g_object_unref(f_stream);
     pthread_mutex_unlock(&gmime_lock);
 
     return (0);
@@ -134,7 +144,7 @@ int
 arg_split(char ***list, char *str)
 {
 
-    char *p, *tmpp, **tmplist;
+    char *p = NULL, *tmpp = NULL, **tmplist = NULL;
     int len = 0, cnt = 0;
     *list = NULL;
 
@@ -220,6 +230,7 @@ sendmail(struct mailzip *mz, struct config *cfg, struct rcptinfo *info, char *se
     pid_t	pid;
     struct rcptaddr *p = info->rcptlist;
 
+
     /* count sendmail command option */
     num = arg_split(&list, cfg->cf_sendmailcommand);
     if (num == 0) {
@@ -232,7 +243,6 @@ sendmail(struct mailzip *mz, struct config *cfg, struct rcptinfo *info, char *se
 	DEBUGLOG("rcpt_addr: %s\n", (p + cnt)->rcpt_addr);
 	
     }
-    p = info->rcptlist;
 
     arg_list = (char **)malloc(sizeof(char *) * (num + 4 + cnt));
     if (arg_list == NULL) {
@@ -279,6 +289,7 @@ sendmail(struct mailzip *mz, struct config *cfg, struct rcptinfo *info, char *se
     }
     arg_list[num + 3 + cnt] = NULL;
 
+
     /*
      * make pipe
      *
@@ -300,9 +311,12 @@ sendmail(struct mailzip *mz, struct config *cfg, struct rcptinfo *info, char *se
 	int	sts = 0;
 
 	parent(fd, mz);
-	if (wait(&sts) < 0) {
+        /* parent process */
+        ret = waitpid(pid, &sts, WUNTRACED);
+        if (ret == -1) {
 	    log(ERR_WAIT_CHILD, "sendmail", strerror(errno));
-	}
+            return -1;
+        }
 	if (WIFEXITED(sts)) {
 	    ret = WEXITSTATUS(sts);
 	    if (ret != 0) {
@@ -312,6 +326,7 @@ sendmail(struct mailzip *mz, struct config *cfg, struct rcptinfo *info, char *se
         	return(SENDMAIL_FAILED);
 	    }
 	} else {
+	    log(ERR_WAIT_CHILD, "sendmail", strerror(errno));
 	    arg_list_free(arg_list, num + 3 + cnt);
 	    free(list);
 	    return(SENDMAIL_FAILED);
@@ -320,15 +335,11 @@ sendmail(struct mailzip *mz, struct config *cfg, struct rcptinfo *info, char *se
 	log(ERR_FORK_CREATE, "sendmail", strerror(errno));
 	arg_list_free(arg_list, num + 3 + cnt);
 	free(list);
-	close(fd[R]);
-	close(fd[W]);
 	return(SENDMAIL_FAILED);
     }
 
     arg_list_free(arg_list, num + 3 + cnt);
     free(list);
-    close(fd[R]);
-    close(fd[W]);
     return(SENDMAIL_SUCCESS);
 }
 
@@ -432,8 +443,10 @@ passwd_sendmail(struct config *cfg, char *rcpt, char *message)
         int     sts = 0;
 
         passwd_parent(fd, message);
-        if (wait(&sts) < 0) {
+        ret = waitpid(pid, &sts, WUNTRACED);
+        if (ret == -1) {
 	    log(ERR_WAIT_CHILD, "passwd_sendmail", strerror(errno));
+            return(SENDMAIL_FAILED);
         }
         if (WIFEXITED(sts)) {
             ret = WEXITSTATUS(sts);
@@ -444,6 +457,7 @@ passwd_sendmail(struct config *cfg, char *rcpt, char *message)
         	return(SENDMAIL_FAILED);
 	    }
         } else {
+	    log(ERR_WAIT_CHILD, "passwd_sendmail", strerror(errno));
             arg_list_free(arg_list, num + 2);
             free(list);
             return(SENDMAIL_FAILED);
@@ -452,14 +466,10 @@ passwd_sendmail(struct config *cfg, char *rcpt, char *message)
 	log(ERR_FORK_CREATE, "passwd_sendmail", strerror(errno));
         arg_list_free(arg_list, num + 2);
         free(list);
-        close(fd[R]);
-        close(fd[W]);
         return(SENDMAIL_FAILED);
     }
 
     arg_list_free(arg_list, num + 2);
     free(list);
-    close(fd[R]);
-    close(fd[W]);
     return(SENDMAIL_SUCCESS);
 }
