@@ -20,8 +20,8 @@
 
 /*
  * $RCSfile: zipconv.c,v $
- * $Revision: 1.9 $
- * $Date: 2009/08/31 10:01:41 $
+ * $Revision: 1.15 $
+ * $Date: 2010/04/10 03:01:49 $
  */
 
 #include <stdio.h>
@@ -49,11 +49,6 @@
 #include "global.h"
 
 static void zip_child(char **, char *, char *, char *);
-static int enc_policydb(SMFICTX *, struct config *, char *, struct rcptaddr *, 
-			struct rcptinfo **, struct rcptinfo **, struct rcptinfo **);
-static int add_enclist(struct rcptinfo ***, char *, char *, char *);
-
-pthread_mutex_t pw_lock = PTHREAD_MUTEX_INITIALIZER;
 pthread_mutex_t gmime_lock = PTHREAD_MUTEX_INITIALIZER;
 
 void
@@ -78,9 +73,7 @@ free_rcptinfo(struct rcptinfo *info)
     if (info != NULL) {
         for (p = info; p != NULL; p = tmpp) {
 	    tmpp = p->Next;
-	    if (p->passwd != NULL) {
-	        free(p->passwd);
-	    }
+	    free(p->passwd);
 	    if (p->rcptlist != NULL) {
 	        free_rcptlist(p->rcptlist);
 	    }
@@ -90,8 +83,7 @@ free_rcptinfo(struct rcptinfo *info)
 }
 
 void
-mailzip_clean(struct mailzip mz, struct rcptinfo *addrmatch, struct rcptinfo *dommatch, 
-	      struct rcptinfo *other)
+mailzip_clean(struct mailzip mz)
 {
     /* free mailzip structure */
     if (mz.zipdir != NULL) {
@@ -122,73 +114,54 @@ mailzip_clean(struct mailzip mz, struct rcptinfo *addrmatch, struct rcptinfo *do
     if (mz.attachfilename != NULL) {
         free(mz.attachfilename);
     }
-
-    /* free rcptinfo structure */
-    free_rcptinfo(addrmatch);
-    free_rcptinfo(dommatch);
-    free_rcptinfo(other);
 }
 
 extern int
 zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
-                 char *from, struct rcptaddr *rcptlist)
+                 char *from, struct rcptinfo *passlist, struct rcptinfo *rdmpasslist)
 {
     struct mailzip mz;
-    struct rcptinfo *addrmatch = NULL, *dommatch = NULL, *other = NULL;
     struct rcptinfo *p;
     int i, ret;
     char *temp = NULL, *notify_message, *rcptaddr;
 
     memset(&mz, 0, sizeof(struct mailzip));
 
-    /* policy check */
-    if (enc_policydb(ctx, cfg, from, rcptlist, &addrmatch, &dommatch, &other) != 0) {
-	mailzip_clean(mz, addrmatch, dommatch, other);
-	return ZIP_CONVERT_FAILED;
-    }
-
-    /* Not policy match address and domain */
-    if ((addrmatch == NULL) && (dommatch == NULL) && (other == NULL)) {
-	DEBUGLOG("Not policy match address and domain");
-	mailzip_clean(mz, addrmatch, dommatch, other);
-	return ZIP_CONVERT_ACCEPT;
-    }
-
     /* parse mail */
     ret = parse_mail(minfo, cfg, &mz);
-    if (ret == PM_NO_MULTIPART) {
-	mailzip_clean(mz, addrmatch, dommatch, other);
+    if ((ret == PM_NO_MULTIPART) || (ret == PM_NO_ATTACHFILE)) {
+	mailzip_clean(mz);
 	return ZIP_CONVERT_ACCEPT;
     } else if (ret == PM_FAILED) {
-	mailzip_clean(mz, addrmatch, dommatch, other);
+	mailzip_clean(mz);
 	return ZIP_CONVERT_FAILED;
     }
 
     if (mk_encpath(cfg, &mz) == -1) {
-	mailzip_clean(mz, addrmatch, dommatch, other);
+	mailzip_clean(mz);
 	return ZIP_CONVERT_FAILED;
     }
 
-    if (addrmatch != NULL) {
-	for (p = addrmatch; p != NULL; p = p->Next) {
-	    /* encription */
+    if (passlist != NULL) {
+	for (p = passlist; p != NULL; p = p->Next) {
+	    /* encryption */
 	    if (convert_zip(cfg, &mz, &p) != 0) {
-		mailzip_clean(mz, addrmatch, dommatch, other);
+		mailzip_clean(mz);
 		return ZIP_CONVERT_FAILED;
 	    }
 	    /* add attach file */
 	    if (add_attach_file(&mz, p) != 0) {
-		mailzip_clean(mz, addrmatch, dommatch, other);
+		mailzip_clean(mz);
 		return ZIP_CONVERT_FAILED;
 	    }
     	    /* send mail */
 	    if (sendmail(&mz, cfg, p, from) != 0) {
-		mailzip_clean(mz, addrmatch, dommatch, other);
+		mailzip_clean(mz);
 		return ZIP_CONVERT_FAILED;
 	    }
 	    if (unlink(mz.encfilepath) == -1) {
 		log(ERR_FILE_REMOVE, "convert_zip", unlink(mz.encfilepath));
-		mailzip_clean(mz, addrmatch, dommatch, other);
+		mailzip_clean(mz);
 		return ZIP_CONVERT_FAILED;
 	    }
             for (i = 0; (p->rcptlist + i)->rcpt_addr != NULL; i++) {
@@ -196,93 +169,59 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
                 /* delete from envelope to */
                 if (smfi_delrcpt(ctx, rcptaddr) == MI_FAILURE) {
                     log(ERR_DELETE_ADDRESS, "zip_convert_mail", rcptaddr);
-                    mailzip_clean(mz, addrmatch, dommatch, other);
+		    mailzip_clean(mz);
                     return ZIP_CONVERT_FAILED;
                 }
             }
 	}
     } 
 
-    if (dommatch != NULL) {
-	for (p = dommatch; p != NULL; p = p->Next) {
-	    /* encription */
-	    if (convert_zip(cfg, &mz, &p) != 0) {
-		mailzip_clean(mz, addrmatch, dommatch, other);
-		return ZIP_CONVERT_FAILED;
-	    }
-	    /* add attach file */
-	    if (add_attach_file(&mz, p) != 0) {
-		mailzip_clean(mz, addrmatch, dommatch, other);
-		return ZIP_CONVERT_FAILED;
-	    }
-    	    /* send mail */
-	    if (sendmail(&mz, cfg, p, from) != 0) {
-		mailzip_clean(mz, addrmatch, dommatch, other);
-		return ZIP_CONVERT_FAILED;
-	    }
-	    if (unlink(mz.encfilepath) == -1) {
-		log(ERR_FILE_REMOVE, "convert_zip", unlink(mz.encfilepath));
-		mailzip_clean(mz, addrmatch, dommatch, other);
-		return ZIP_CONVERT_FAILED;
-	    }
-            for (i = 0; (p->rcptlist + i)->rcpt_addr != NULL; i++) {
-                rcptaddr = (p->rcptlist + i)->rcpt_addr;
-                /* delete from envelope to */
-                if (smfi_delrcpt(ctx, rcptaddr) == MI_FAILURE) {
-                    log(ERR_DELETE_ADDRESS, "zip_convert_mail", rcptaddr);
-                    mailzip_clean(mz, addrmatch, dommatch, other);
-                    return ZIP_CONVERT_FAILED;
-                }
-            }
-	}
-    } 
-
-    if (other != NULL) {
-	/* encription */
-	if (convert_zip(cfg, &mz, &other) != 0) {
-	    mailzip_clean(mz, addrmatch, dommatch, other);
+    if (rdmpasslist != NULL) {
+	/* encryption */
+	if (convert_zip(cfg, &mz, &rdmpasslist) != 0) {
+	    mailzip_clean(mz);
 	    return ZIP_CONVERT_FAILED;
 	}
 	/* add attach file */
-	if (add_attach_file(&mz, other) != 0) {
-	    mailzip_clean(mz, addrmatch, dommatch, other);
+	if (add_attach_file(&mz, rdmpasslist) != 0) {
+	    mailzip_clean(mz);
 	    return ZIP_CONVERT_FAILED;
 	}
 
     	/* send mail */
-	if (sendmail(&mz, cfg, other, from) != 0) {
-	    mailzip_clean(mz, addrmatch, dommatch, other);
+	if (sendmail(&mz, cfg, rdmpasslist, from) != 0) {
+	    mailzip_clean(mz);
 	    return ZIP_CONVERT_FAILED;
 	}
 
 	temp = tmpl_init();
-	if (tmpl_tag_replace(temp, mz, other, from, &notify_message) != 0){
-	    mailzip_clean(mz, addrmatch, dommatch, other);
+	if (tmpl_tag_replace(temp, mz, rdmpasslist, from, &notify_message) != 0){
+	    mailzip_clean(mz);
 	    return ZIP_CONVERT_FAILED;
 	}
 	tmpl_release(temp);
 
 	if (passwd_sendmail(cfg, from, notify_message) != 0) {
 	    free(notify_message);
-	    mailzip_clean(mz, addrmatch, dommatch, other);
+	    mailzip_clean(mz);
 	    return ZIP_CONVERT_FAILED;
 	}
 	free(notify_message);
-        for (i = 0; (other->rcptlist + i)->rcpt_addr != NULL; i++) {
-            rcptaddr = (other->rcptlist + i)->rcpt_addr;
+        for (i = 0; (rdmpasslist->rcptlist + i)->rcpt_addr != NULL; i++) {
+            rcptaddr = (rdmpasslist->rcptlist + i)->rcpt_addr;
             /* delete from envelope to */
             if (smfi_delrcpt(ctx, rcptaddr) == MI_FAILURE) {
                 log(ERR_DELETE_ADDRESS, "zip_convert_mail", rcptaddr);
-                mailzip_clean(mz, addrmatch, dommatch, other);
+	        mailzip_clean(mz);
                 return ZIP_CONVERT_FAILED;
             }
         }
     } 
 
 #ifdef	DEBUG
-    if (addrmatch != NULL) {
-	log("------------- addrmatch list start -------------");
-        for (p = addrmatch; p != NULL; p = p->Next) {
+    if (passlist != NULL) {
+	log("------------- passlist list start -------------");
+        for (p = passlist; p != NULL; p = p->Next) {
 	    log("keyword: %s", p->keyword);
 	    log("keyword_len: %d", p->keyword_len);
 	    log("passwd: %s", p->passwd);
@@ -291,11 +230,11 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
 	    }
 	    log("encfilepath: %s", mz.encfilepath);
         }
-	log("-------------- addrmatch list end --------------");
+	log("-------------- passlist list end --------------");
     }
-    if (dommatch != NULL) {
-	log("------------- dommatch list start -------------");
-        for (p = dommatch; p != NULL; p = p->Next) {
+    if (rdmpasslist != NULL) {
+	log("------------- rdmpasslist list start -------------");
+        for (p = rdmpasslist; p != NULL; p = p->Next) {
 	    log("keyword: %s", p->keyword);
 	    log("keyword_len: %d", p->keyword_len);
 	    log("passwd: %s", p->passwd);
@@ -304,24 +243,11 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
 	    }
 	    log("encfilepath: %s", mz.encfilepath);
         }
-	log("-------------- dommatch list end --------------");
-    }
-    if (other != NULL) {
-	log("------------- other list start -------------");
-        for (p = other; p != NULL; p = p->Next) {
-	    log("keyword: %s", p->keyword);
-	    log("keyword_len: %d", p->keyword_len);
-	    log("passwd: %s", p->passwd);
-	    for (i = 0; (p->rcptlist + i)->rcpt_addr != NULL; i++) {
-	        log("rcptaddr: %s", (p->rcptlist + i)->rcpt_addr);
-	    }
-	    log("encfilepath: %s", mz.encfilepath);
-        }
-	log("-------------- other list end --------------");
+	log("-------------- rdmpasslist list end --------------");
     }
 #endif /* DEBUG */
 
-    mailzip_clean(mz, addrmatch, dommatch, other);
+    mailzip_clean(mz);
     return ZIP_CONVERT_SUCCESS;
 }
 
@@ -342,45 +268,35 @@ add_attach_file(struct mailzip *mz, struct rcptinfo *rcpt)
 
     pthread_mutex_lock(&gmime_lock);
     object = g_mime_message_get_mime_part(mz->message);
-    pthread_mutex_unlock(&gmime_lock);
 
     if (GMIME_IS_MULTIPART (object)) {
 
         index = g_mime_multipart_get_number((GMimeMultipart *)object);
 
         for (i = index - 1; i > 0; i--) {
-            pthread_mutex_lock(&gmime_lock);
             att = g_mime_multipart_get_part((GMimeMultipart *)object, i);
-            pthread_mutex_unlock(&gmime_lock);
             filename = g_mime_part_get_filename((GMimePart *)att);
             if (filename == NULL) {
-                pthread_mutex_lock(&gmime_lock);
                 g_object_unref (att);
-                pthread_mutex_unlock(&gmime_lock);
                 att = NULL;
                 continue;
             }
 
             g_mime_multipart_remove_part((GMimeMultipart *)object, att);
-            pthread_mutex_lock(&gmime_lock);
             g_object_unref (att);
-            pthread_mutex_unlock(&gmime_lock);
         }
     }
 
     fp = fopen(mz->encfilepath, "r");
     if (fp == NULL) {
 	log(ERR_FILE_OPEN, "add_attach_file", mz->encfilepath);
-        pthread_mutex_lock(&gmime_lock);
         g_object_unref (object);
         pthread_mutex_unlock(&gmime_lock);
 	return -1;
     }
 
     /* create part object */
-    pthread_mutex_lock(&gmime_lock);
     part = g_mime_part_new();
-    pthread_mutex_unlock(&gmime_lock);
 
     /* set part object */
     g_mime_part_set_content_header(part, CONTENTTYPE_HEADER, CONTENTTYPE_ZIP);
@@ -389,81 +305,33 @@ add_attach_file(struct mailzip *mz, struct rcptinfo *rcpt)
     g_mime_part_set_content_disposition(part, CONTENTDISPOSITION);
 
     /* open stream */
-    pthread_mutex_lock(&gmime_lock);
     r_stream = g_mime_stream_file_new(fp); 
-    pthread_mutex_unlock(&gmime_lock);
 
     /* create filter object */
-    pthread_mutex_lock(&gmime_lock);
     filter = g_mime_filter_basic_new_type(GMIME_FILTER_BASIC_BASE64_ENC);
-    pthread_mutex_unlock(&gmime_lock);
 
     /* set filter */
-    pthread_mutex_lock(&gmime_lock);
     f_stream = g_mime_stream_filter_new_with_stream(r_stream);
     g_object_unref (r_stream);
-    pthread_mutex_unlock(&gmime_lock);
     g_mime_stream_filter_add((GMimeStreamFilter *)f_stream, filter);
 
     /* create wrapper object */
-    pthread_mutex_lock(&gmime_lock);
     wrapper = g_mime_data_wrapper_new_with_stream(f_stream, GMIME_PART_ENCODING_BASE64);
-    pthread_mutex_unlock(&gmime_lock);
 
     /* set part */
     g_mime_part_set_content_object(part, wrapper);
     g_mime_multipart_add_part((GMimeMultipart *)object, (GMimeObject *)part);
 
-    pthread_mutex_lock(&gmime_lock);
     g_object_unref (filter);
     g_object_unref (part);
     g_object_unref (wrapper);
     g_object_unref (f_stream);
     g_object_unref (object);
+
     pthread_mutex_unlock(&gmime_lock);
-
     return 0;
 }
 
-char seed[] = SEED;
-/*
- * mk_passwd
- *
- * Args:
- *	char	**passwd	passwd string pointer
- *	int	  count		passwd length
- *
- * Returns:
- *	PASSWD_SUCCESS		Success
- *	PASSWD_FAILED		Failed
- */
-int
-mk_passwd(char **passwd, int count)
-{
-    long val;
-    int i;
-
-    *passwd = (char *)malloc(sizeof(char *) * count + 1);
-    if (*passwd == NULL) {
-	log(ERR_MEMORY_ALLOCATE, "mk_passwd", "passwd", strerror(errno));
-	return(PASSWD_FAILED);
-    }
-
-    pthread_mutex_lock(&pw_lock);
-
-    for(i = 0; i < count; i++) {
-        val = lrand48() % (sizeof(seed) - 1);
-
-        *(*(passwd) + i) = seed[val];
-
-    }
-    *(*(passwd) + count) = '\0';
-
-    pthread_mutex_unlock(&pw_lock);
-
-    return(PASSWD_SUCCESS);
-}
-
 /*
  * mk_new_filename
  *
@@ -687,55 +555,49 @@ parse_mail(struct mailinfo *minfo, struct config *cfg, struct mailzip *mz)
     GMimeStream *r_stream;
     GMimeParser *parser;
     GMimeObject *object;
+    const GMimeContentType *ct_object = NULL;
     FILE *rfp;
     char *tmpdir, *tmpstr;
     int rfd, len;
 
+    pthread_mutex_lock(&gmime_lock);
+
     /* create object */
     if(minfo->ii_status & MS_STATUS_FILE) {
 	DEBUGLOG("Create message object from file");
         /* stream */
 	if ((rfd = dup(minfo->ii_fd)) < 0) {
 	    log(ERR_FILE_OPEN, "parse_mail", minfo->ii_fd);
+	    pthread_mutex_unlock(&gmime_lock);
 	    return(PM_FAILED);
 	}
         rfp = fdopen(rfd, "r");
         if (rfp == NULL) {
 	    log(ERR_FILE_OPEN, "parse_mail", minfo->ii_fd);
+	    pthread_mutex_unlock(&gmime_lock);
 	    return(PM_FAILED);
         }
         fseek(rfp, 0L, SEEK_SET);
 
-        pthread_mutex_lock(&gmime_lock);
         r_stream = g_mime_stream_file_new (rfp);
-        pthread_mutex_unlock(&gmime_lock);
     } else {
 	DEBUGLOG("Create message object from memory");
-        pthread_mutex_lock(&gmime_lock);
 	r_stream = g_mime_stream_mem_new_with_buffer (minfo->ii_mbuf, minfo->ii_len);
-        pthread_mutex_unlock(&gmime_lock);
     }
 
     /* parser */
-    pthread_mutex_lock(&gmime_lock);
     parser = g_mime_parser_new_with_stream(r_stream);
     g_object_unref (r_stream);
-    pthread_mutex_unlock(&gmime_lock);
 
     /* gmime object */
-    pthread_mutex_lock(&gmime_lock);
     mz->message = g_mime_parser_construct_message(parser);
-    pthread_mutex_unlock(&gmime_lock);
     if (mz->message == NULL) {
 	log(ERR_GMIME, "parse_mail", "g_mime_parser_construct_message");
-        pthread_mutex_lock(&gmime_lock);
         g_object_unref (parser);
-        pthread_mutex_unlock(&gmime_lock);
+	pthread_mutex_unlock(&gmime_lock);
 	return(PM_FAILED);
     }
-    pthread_mutex_lock(&gmime_lock);
     g_object_unref (parser);
-    pthread_mutex_unlock(&gmime_lock);
 
     /* Add header */
     g_mime_message_add_header(mz->message, XHEADERNAME, XHEADERVALUE);
@@ -746,26 +608,30 @@ parse_mail(struct mailinfo *minfo, struct config *cfg, struct mailzip *mz)
     mz->date = g_mime_message_get_date_string(mz->message);
 
     /* gmime part object */
-    pthread_mutex_lock(&gmime_lock);
     object = g_mime_message_get_mime_part(mz->message);
-    pthread_mutex_unlock(&gmime_lock);
 
     if (!GMIME_IS_MULTIPART (object)) {
-        pthread_mutex_lock(&gmime_lock);
         g_object_unref (object);
-        pthread_mutex_unlock(&gmime_lock);
+	pthread_mutex_unlock(&gmime_lock);
 	return(PM_NO_MULTIPART);
     }
 
+    ct_object = g_mime_object_get_content_type(object);
+    if ((strncmp(ct_object->type, CONTENT_T_MULTIPART, LEN_CONTENT_T_MULTIPART) == 0) && 
+	(strncmp(ct_object->subtype, CONTENT_SUBT_RELATED, LEN_CONTENT_SUBT_RELATED) == 0)) {
+        g_object_unref (object);
+	pthread_mutex_unlock(&gmime_lock);
+	return(PM_NO_ATTACHFILE);
+    }
+
     /* make zip directory */
     len = strlen(cfg->cf_encryptiontmpdir) + strlen(TMPDIR_TMPL) + 2;
     tmpstr = (char *)malloc(len);
 
     if (tmpstr == NULL) {
 	log(ERR_MEMORY_ALLOCATE, "parse_mail", "tmpstr", strerror(errno));
-        pthread_mutex_lock(&gmime_lock);
         g_object_unref (object);
-        pthread_mutex_unlock(&gmime_lock);
+	pthread_mutex_unlock(&gmime_lock);
 	return(PM_FAILED);
     }
     snprintf(tmpstr, len, FILE_PATH, cfg->cf_encryptiontmpdir, TMPDIR_TMPL);
@@ -773,39 +639,36 @@ parse_mail(struct mailinfo *minfo, struct config *cfg, struct mailzip *mz)
     tmpdir = mkdtemp(tmpstr);
     if (tmpdir == NULL) {
         log(ERR_DIRECTORY_MAKE, "parse_mail", tmpstr, strerror(errno));
-        pthread_mutex_lock(&gmime_lock);
         g_object_unref (object);
-        pthread_mutex_unlock(&gmime_lock);
 	free(tmpstr);
+	pthread_mutex_unlock(&gmime_lock);
 	return(PM_FAILED);
     }
     mz->zipdir = strdup(tmpdir);
     if (mz->zipdir == NULL) {
 	log(ERR_MEMORY_ALLOCATE, "parse_mail", "mz->zipdir", strerror(errno));
-        pthread_mutex_lock(&gmime_lock);
         g_object_unref (object);
-        pthread_mutex_unlock(&gmime_lock);
 	free(tmpstr);
+	pthread_mutex_unlock(&gmime_lock);
 	return -1;
     }
     free(tmpstr);
 
     if (drop_file(object, cfg, mz) != 0) {
-        pthread_mutex_lock(&gmime_lock);
         g_object_unref (object);
-        pthread_mutex_unlock(&gmime_lock);
+	pthread_mutex_unlock(&gmime_lock);
 	return -1;
     }
 
-    pthread_mutex_lock(&gmime_lock);
     g_object_unref (object);
-    pthread_mutex_unlock(&gmime_lock);
 
     /* Not attach file */
     if (mz->namelist == NULL) {
+	pthread_mutex_unlock(&gmime_lock);
         return(PM_NO_MULTIPART);
     }
 
+    pthread_mutex_unlock(&gmime_lock);
     return(PM_SUCCESS);
 }
 
@@ -829,17 +692,13 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
     index = g_mime_multipart_get_number((GMimeMultipart *)object);
 
     for (i = index - 1; i > 0; i--) {
-        pthread_mutex_lock(&gmime_lock);
 	att = g_mime_multipart_get_part((GMimeMultipart *)object, i);
-        pthread_mutex_unlock(&gmime_lock);
 
 	if (GMIME_IS_MESSAGE_PART(att)) {
 	    filename = MESSAGE_FILENAME;
 	} else if (GMIME_IS_MULTIPART (att)) {
 	    ret = drop_file(att, cfg, mz);
-	    pthread_mutex_lock(&gmime_lock);
 	    g_object_unref (att);
-	    pthread_mutex_unlock(&gmime_lock);
 	    if (ret < 0) {
 		return -1;
 	    } 
@@ -849,9 +708,7 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
 	    /* get filename */
 	    filename = g_mime_part_get_filename((GMimePart *)att);
 	    if (filename == NULL) {
-                pthread_mutex_lock(&gmime_lock);
 	        g_object_unref (att);
-                pthread_mutex_unlock(&gmime_lock);
 	        continue;
 	    }
 	}
@@ -866,18 +723,14 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
 	tmpname = strdup(filename);
 	if (tmpname == NULL) {
 	    log(ERR_MEMORY_ALLOCATE, "drop_file", "tmpname", strerror(errno));
-            pthread_mutex_lock(&gmime_lock);
 	    g_object_unref (att);
-            pthread_mutex_unlock(&gmime_lock);
 	    return -1;
         }
 	
 	/* convert str code */
 	if (dg_str2code(tmpname, &tmpfilename, "UTF8", cfg->cf_strcode) != 0) {
 	    log(ERR_CHARCODE_CONVERT, "drop_file");
-            pthread_mutex_lock(&gmime_lock);
             g_object_unref (att);
-            pthread_mutex_unlock(&gmime_lock);
 	    free(tmpname);
 	    return -1;
 	}
@@ -886,9 +739,7 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
         mk_new_filename(newfilename, tmpfilename, mz->zipdir, count);
 
         if (push_attnamelist(&(mz->namelist), newfilename) != 0) {
-            pthread_mutex_lock(&gmime_lock);
             g_object_unref (att);
-            pthread_mutex_unlock(&gmime_lock);
 	    free(tmpfilename);
             return -1;
         }
@@ -900,56 +751,38 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
 	wfp = fopen(tmppath, "w");
 	if (wfp == NULL) {
 	    log(ERR_FILE_OPEN, "drop_file", tmppath);
-            pthread_mutex_lock(&gmime_lock);
 	    g_object_unref (att);
-            pthread_mutex_unlock(&gmime_lock);
 	    return -1;
 	}
 
-        pthread_mutex_lock(&gmime_lock);
 	w_stream = g_mime_stream_file_new (wfp);
-        pthread_mutex_unlock(&gmime_lock);
 
 	if (GMIME_IS_MESSAGE_PART(att)) {
-            pthread_mutex_lock(&gmime_lock);
 	    message = g_mime_message_part_get_message((GMimeMessagePart *)att);
-            pthread_mutex_unlock(&gmime_lock);
 	    ret = g_mime_message_write_to_stream (message, w_stream);
-            pthread_mutex_lock(&gmime_lock);
             g_object_unref (message);
             g_object_unref (w_stream);
-            pthread_mutex_unlock(&gmime_lock);
 	} else {
-            pthread_mutex_lock(&gmime_lock);
 	    wrapper = g_mime_part_get_content_object((GMimePart *)att);
-            pthread_mutex_unlock(&gmime_lock);
 	    if (wrapper == NULL) {
 	        log(ERR_GMIME, "drop_file", "g_mime_part_get_content_object");
-                pthread_mutex_lock(&gmime_lock);
 	        g_object_unref (att);
                 g_object_unref (w_stream);
-                pthread_mutex_unlock(&gmime_lock);
 	        return -1;
 	    }
 
 	    ret = g_mime_data_wrapper_write_to_stream(wrapper, w_stream);
-            pthread_mutex_lock(&gmime_lock);
             g_object_unref (wrapper);
             g_object_unref (w_stream);
-            pthread_mutex_unlock(&gmime_lock);
 	}
 	if (ret < 0) {
 	    log(ERR_GMIME, "drop_file", "g_mime_data_wrapper_write_to_stream");
-            pthread_mutex_lock(&gmime_lock);
 	    g_object_unref (att);
-            pthread_mutex_unlock(&gmime_lock);
 	    return -1;
 	}
 
         g_mime_multipart_remove_part((GMimeMultipart *)object, att);
-        pthread_mutex_lock(&gmime_lock);
         g_object_unref (att);
-        pthread_mutex_unlock(&gmime_lock);
     }
 
     return 0;
@@ -968,7 +801,7 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
  * Argument
  *      struct config  *cfg	config structure
  *      char *tmpdir		temporary directory name
- *      char *passwd		encription password
+ *      char *passwd		encryption password
  *
  * Return value
  *      0       Normal end.
@@ -995,19 +828,22 @@ convert_zip(struct config *cfg, struct mailzip *mz, struct rcptinfo **rcpt)
 	/* child process */
 	zip_child(list, cfg->cf_zipcommandopt, mz->zipdir, (*rcpt)->passwd);
     } else if (pid > 0) {
-	/* parent process */
-	if (wait(&sts) < 0) {
+        /* parent process */
+        ret = waitpid(pid, &sts, WUNTRACED);
+	if (ret == -1) {
             log(ERR_WAIT_CHILD, "convert_zip", strerror(errno));
+            return -1;
 	}
-	if (WIFEXITED(sts)) {       /* return or exit */
-	    ret = WEXITSTATUS(sts); /* exit code */
-	    if (ret != 0) {
-        	log(ERR_ZIP_CONVERT, "convert_zip", mz->encfilepath);
-	        return -1;
-	    }
-	} else {
-	    return -1;
-	}
+        if (WIFEXITED(sts)) {       /* return or exit */
+            ret = WEXITSTATUS(sts); /* exit code */
+            if (ret != 0) {
+                log(ERR_ZIP_CONVERT, "convert_zip", mz->encfilepath);
+                return -1;
+            }
+        } else {
+            log(ERR_WAIT_CHILD, "convert_zip", strerror(errno));
+            return -1;
+        }
     } else {
 	/* make process failed */
         log(ERR_FORK_CREATE, "convert_zip", strerror(errno));
@@ -1041,7 +877,7 @@ zip_child(char **list, char *commandopt, char *dir, char *passwd)
     }
     free(envstr);
 
-    /* chage encription directory */
+    /* chage encryption directory */
     if (chdir(dir) != 0) {
         log(ERR_DIRECTORY_CHANGE, "zip_child", dir);
         exit(1); 
@@ -1060,270 +896,6 @@ zip_child(char **list, char *commandopt, char *dir, char *passwd)
     exit(0);
 }
 
-#define ATMARK	'@'
-#define OTHERKEYWORD	"other"
-static int
-enc_policydb(SMFICTX *ctx, struct config *cfg, char *from, struct rcptaddr *rcptlist, 
-	     struct rcptinfo **addrmatch, struct rcptinfo **dommatch, 
-	     struct rcptinfo **other)
-{
-    DB *sender_dbp, *rcpt_dbp;
-    int i, ret;
-    int sender_flg = NOT_FOUND;
-    char *passwd = NULL;
-    char *patmark;
-    char *rcptaddr;
-    int rcptaddrlen = 0;
-    int pass_len = cfg->cf_passwordlength;
-    char *onetime_pass = NULL;
-    struct rcptinfo *p = NULL;
-    int dom_flg = NOT_FOUND;
-
-    /* make onetime password */
-    if (mk_passwd(&onetime_pass, pass_len) != PASSWD_SUCCESS) {
-	return -1;
-    }
-    DEBUGLOG("Make password ok");
-
-    DEBUGLOG("Sender check start");
-
-    /* senderdb open */
-    ret = db_open(&sender_dbp, cfg->cf_senderdbpath, cfg->cf_senderdbtype);
-    if (ret == DB_ERROR) {
-	free(onetime_pass);
-	return -1;
-    }
-
-    /* search sender address */
-    ret = db_search(sender_dbp, cfg->cf_senderdbpath, from, &passwd);
-    if (ret == DB_ERROR) {
-	free(onetime_pass);
-	db_close(sender_dbp);
-	return -1;
-    }
-    if (ret == FOUND) {
-        DEBUGLOG("Sender address found.(%s)", from);
-	sender_flg = FOUND;
-	if (passwd != NULL) {
-            free(passwd);
-            passwd = NULL;
-	}
-    }
-
-    /* search sender domain */
-    if (sender_flg == NOT_FOUND) {
-	patmark = strchr(from, ATMARK);
-	if (patmark != NULL) {
-	    /* patmark next is domain */
-	    ret = db_search(sender_dbp, cfg->cf_senderdbpath, patmark + 1, &passwd);
-	    if (ret == DB_ERROR) {
-		free(onetime_pass);
-		db_close(sender_dbp);
-		return -1;
-	    }
-	    if (ret == FOUND) {
-            	DEBUGLOG("Sender domain found.(%s)", patmark + 1);
-		sender_flg = FOUND;
-	    	if (passwd != NULL) {
-		    free(passwd);
-		    passwd = NULL;
-		}
-	    }
-        }
-    }
-
-    /* senderdb close */
-    db_close(sender_dbp);
-
-#ifdef	DEBUG
-    if (sender_flg == NOT_FOUND) {
-        log("sender not found");
-    }
-#endif /* DEBUG */
-
-    /* rcptdb open */
-    if (db_open(&rcpt_dbp, cfg->cf_rcptdbpath, cfg->cf_rcptdbtype) == DB_ERROR) {
-	free(onetime_pass);
-	return -1;
-    }
-
-    for (i = 0; (rcptlist + i)->rcpt_addr != NULL; i++) {
-	rcptaddr = (rcptlist + i)->rcpt_addr;
-	rcptaddrlen = (rcptlist + i)->rcpt_addr_len;
-
-	/* search rcpt address */
-	ret = db_search(rcpt_dbp, cfg->cf_rcptdbpath, rcptaddr, &passwd);
-	if (ret == DB_ERROR) {
-	    free(onetime_pass);
-	    db_close(sender_dbp);
-	    return -1;
-	}
-	if (ret == FOUND) {
-	    if (passwd != NULL) {
-	        /* Add address list */
-	        if (add_enclist(&addrmatch, rcptaddr, rcptaddr, passwd) != 0) {
-		    free(onetime_pass);
-		    db_close(sender_dbp);
-		    return -1;
-		}	
-            	DEBUGLOG("Add keyword to addrmatch structure.(%s)", rcptaddr);
-		if (passwd != NULL) {
-	            free(passwd);
-	            passwd = NULL;
-		}
-	    } else {
-	        /* Add other list */
-	        if (add_enclist(&other, OTHERKEYWORD, rcptaddr, onetime_pass) != 0) {
-		    free(onetime_pass);
-		    db_close(sender_dbp);
-		    return -1;
-		}
-            	DEBUGLOG("Add keyword to other structure.(%s)", rcptaddr);
-	    }
-	    continue;
-	}
-
-	/* search rcpt domain */
-	patmark = strchr(rcptaddr, ATMARK);
-	if (patmark != NULL) {
-	    /* search domain list */
-	    if (*dommatch != NULL) {
-	        for (p = *dommatch; p != NULL; p = p->Next) {
-	    	    if (strncmp(p->keyword, patmark + 1, p->keyword_len) != 0) {
-			continue;
-	    	    }
-	            /* keyword found from rcptinfo list */
-	            if (push_rcptlist(&(p->rcptlist), rcptaddr) != 0) {
-			free(onetime_pass);
-			db_close(sender_dbp);
-		   	return -1;
-	            }
-            	    DEBUGLOG("Add recpient to rcptlist.(%s)", rcptaddr);
-		    /* domain found flg on */
-		    dom_flg = FOUND;
-	        }
-
-		/* domain found from dommatch list */
-		if (dom_flg == FOUND) {
-		    dom_flg = NOT_FOUND;
-		    continue;
-		}
-	    } 
-
-	    /* patmark next is domain */
-	    ret = db_search(rcpt_dbp, cfg->cf_rcptdbpath, patmark + 1, &passwd);
-	    if (ret == DB_ERROR) {
-		free(onetime_pass);
-		db_close(sender_dbp);
-		return -1;
-	    }
-	    if (ret == FOUND) {
-		if (passwd != NULL) {
-	            /* Add domain list */
-	            if (add_enclist(&dommatch, patmark + 1, rcptaddr, passwd) != 0) {
-			free(onetime_pass);
-			db_close(sender_dbp);
-			return -1;
-		    }
-            	    DEBUGLOG("Add keyword to dommatch structure.(%s)", patmark + 1);
-		    if (passwd != NULL) {
-	                free(passwd);
-	                passwd = NULL;
-		    }
-		} else {
-	            /* Add other list */
-	            if (add_enclist(&other, OTHERKEYWORD, rcptaddr, onetime_pass) != 0) {
-			free(onetime_pass);
-			db_close(sender_dbp);
-			return -1;
-		    }
-            	    DEBUGLOG("Add keyword to other structure.(%s)", patmark + 1);
-		}
-	        continue;
-	    }
-	}
-
-	/* found sender address but not found rcpt address */
-	if (sender_flg == FOUND) {
-	    /* Add other list */
-	    if (add_enclist(&other, OTHERKEYWORD, rcptaddr, onetime_pass) != 0) {
-		free(onetime_pass);
-		db_close(sender_dbp);
-		return -1;
-	    }
-            DEBUGLOG("Add keyword to other structure.(%s)", rcptaddr);
-	}
-
-    }
-
-    free(onetime_pass);
-
-    /* rcptdb close */
-    db_close(rcpt_dbp);
-    
-    return 0;
-}
-
-static int
-add_enclist(struct rcptinfo ***head, char *keyword, char *addr, char *passwd)
-{
-    struct rcptinfo *p, *tmpp = NULL, *rcptinfo;
-
-    if (*head != NULL) {
-	for (p = **head; p != NULL; p = p->Next) {
-	    /* set last structure */
-	    tmpp = p;
-
-	    if (strncmp(p->keyword, keyword, p->keyword_len) != 0) {
-		continue;
-	    }
-	    /* keyword found from rcptinfo list */
-	    if (push_rcptlist(&(p->rcptlist), addr) != 0) {
-		return -1;
-	    }
-	    return 0;
-	}
-    } 
-
-    if (passwd == NULL) {
-	log(ERR_PASS_MUST_BE_SET, "add_enclist");
-	return -1;
-    }
-
-    /* create rcptinfo structure */
-    rcptinfo = (struct rcptinfo *)malloc(sizeof(struct rcptinfo));
-    if (rcptinfo == NULL) {
-	log(ERR_MEMORY_ALLOCATE, "add_enclist", "rcptinfo", strerror(errno));
-	return -1;
-    }
-    memset(rcptinfo, 0, sizeof *rcptinfo);
-
-    /* not allocate memory */ 
-    rcptinfo->keyword = keyword;
-    rcptinfo->keyword_len = strlen(keyword);
-
-    /* allocate memory */ 
-    rcptinfo->passwd = strdup(passwd);
-    if (rcptinfo->passwd == NULL) {
-	log(ERR_MEMORY_ALLOCATE, "add_enclist", "rcptinfo->passwd", strerror(errno));
-	return -1;
-    }
-
-    if (push_rcptlist(&(rcptinfo->rcptlist), addr) != 0) {
-	free(rcptinfo);
-	return -1;
-    }
-    rcptinfo->Next = NULL;
-
-    if (**head != NULL) {
-	tmpp->Next = rcptinfo;
-    } else {
-	**head = rcptinfo;
-    }
-
-    return 0;
-}
-
 /*
  * check_ascii
  *
