@@ -20,8 +20,8 @@
 
 /*
  * $RCSfile: zipconv.h,v $
- * $Revision: 1.6 $
- * $Date: 2009/08/07 07:52:29 $
+ * $Revision: 1.9 $
+ * $Date: 2010/04/10 03:02:07 $
  */
 #include <libmilter/mfapi.h>
 #include "mailzip_config.h"
@@ -91,20 +91,21 @@ struct name_list {
 };
 
 int parse_mail(struct mailinfo *, struct config *, struct mailzip *);
-extern int zip_convert_mail(SMFICTX *, struct mailinfo *, struct config *, char *, struct rcptaddr *);
+extern int zip_convert_mail(SMFICTX *, struct mailinfo *, struct config *, char *, struct rcptinfo *, struct rcptinfo *);
 void g_message_callback(GMimeObject *, gpointer);
 int mk_new_filename(char *, const char *, char *, int);
 int remove_file_with_dir_recursive(char *);
 int convert_zip(struct config *, struct mailzip *, struct rcptinfo **);
 int check_ascii(char *);
-int mk_passwd(char **, int);
 int mk_encpath(struct config *, struct mailzip *);
 int add_attach_file(struct mailzip *, struct rcptinfo *);
 int drop_file(GMimeObject *, struct config *, struct mailzip *);
+void free_rcptinfo(struct rcptinfo *);
 
 /* Return Values */
 #define PM_SUCCESS		0
 #define PM_NO_MULTIPART		1
+#define PM_NO_ATTACHFILE	2
 #define PM_FAILED		-1
 #define RM_R_SUCCESS		0
 #define RM_R_NODIR		1
@@ -128,4 +129,9 @@ int drop_file(GMimeObject *, struct config *, struct mailzip *);
 #define ZIP_CONVERT_FAILED	-1
 #define ZIP_CONVERT_ACCEPT	1
 
+#define CONTENT_T_MULTIPART		"multipart"
+#define LEN_CONTENT_T_MULTIPART		sizeof(CONTENT_T_MULTIPART)
+#define CONTENT_SUBT_RELATED		"related"
+#define LEN_CONTENT_SUBT_RELATED	sizeof(CONTENT_SUBT_RELATED)
+
 #endif /* _MAILZIP_ZIPCONV_H_ */
